"""
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        grid_lines.py
Date:           10/15/14
Description:    This script will break selected/all objects into parts based
                on material assignment.
Installation:   Place this file in your maya scripts directory
                (e.g. ..\My Documents\maya\scripts) then run the following
                python commands:
                    import material_break
                    material_break.main()
"""
import maya.cmds as cmds


class MaterialBreak(object):
    def __init__(self):
        self.keep_original_checkbox = None
        self.selection_only_checkbox = None
        self.status_label = None
        self.progress_bar = None
        self.material_break_button = None

    @staticmethod
    def get_duplicate_mesh(mesh_object):
        """Returns a duplicate of 'mesh_object' NOTE: The hierarchy is
        deleted so only the actual 'mesh_object' and its transform exist"""
        copy_objects = cmds.duplicate(mesh_object, renameChildren=True)
        copy_objects = cmds.ls(copy_objects, long=True)

        # delete hierarchy if it exists
        if len(copy_objects) > 1:
            temp = copy_objects.pop(0)
            cmds.delete(copy_objects)
            copy_objects = [temp]

        mesh_copy = cmds.listRelatives(copy_objects[0], shapes=True,
                                       noIntermediate=True, fullPath=True)

        # if no shape node exists, delete the copy and return None
        if not mesh_copy:
            cmds.delete(copy_objects)
            return None

        # if more than one shape node, take the first one only
        if len(mesh_copy) > 1:
            temp = mesh_copy.pop(0)
            cmds.delete(mesh_copy)
            mesh_copy = [temp]

        return mesh_copy[0]

    @staticmethod
    def get_mesh_children(parent_object):
        """Returns a list of all mesh children of 'parent_object' """
        mesh_children = []
        child_shapes = cmds.listRelatives(parent_object, allDescendents=True,
                                          noIntermediate=True, fullPath=True)
        if child_shapes:
            for current in child_shapes:
                if cmds.nodeType(current) == 'mesh':
                    mesh_children.append(current)
        
        return mesh_children

    def get_mesh_objects(self, selection_only):
        """Returns a list of selected/all mesh objects"""
        mesh_objects = []
        
        if selection_only:
            selection = cmds.ls(selection=True, long=True, noIntermediate=True)
            cmds.delete(selection, constructionHistory=True)
            for current in selection:
                if cmds.nodeType(current) == 'transform':
                    mesh_objects.extend(self.get_mesh_children(current))
                if cmds.nodeType(current) == 'mesh':
                    mesh_objects.append(current)
        else:
            mesh_objects = cmds.ls(type='mesh', long=True, noIntermediate=True)
            cmds.delete(mesh_objects, constructionHistory=True)
        
        return mesh_objects
    
    @staticmethod
    def get_unique_shading_groups(mesh_object):
        """Returns a list of unique shading groups attached to mesh
        'mesh_object' """
        unique_connections = []
        all_connections = cmds.listConnections(mesh_object, source=False,
                                               destination=True,
                                               type='shadingEngine')
        if all_connections:
            unique_connections = set(all_connections)
            unique_connections = list(unique_connections)
        
        return unique_connections

    @staticmethod
    def get_mesh_shading_members(shading_group, mesh_object):
        """Returns a list of shading group members for 'shading_group' on mesh
        'mesh_object' """
        mesh_members = []

        if (cmds.polyEvaluate(mesh_object, face=True)) > 0:
            cmds.select(mesh_object + '.f[0]')
            face_name = cmds.ls(selection=True)
            cmds.select(clear=True)
            face_name = face_name[0].split('.f')[0]

            if cmds.sets(shading_group, query=True):
                for current in cmds.sets(shading_group, query=True):
                    if current.startswith(face_name + '.f'):
                        mesh_members.append(current)

        return mesh_members

    def set_material_break(self, mesh_object, shading_groups):
        """Separates 'mesh_object' by 'shading_groups' NOTE: Duplication is
        used for safety. CAD data can be very unpredictable"""
        undo_state = cmds.undoInfo(query=True, state=True)
        cmds.undoInfo(state=False)

        for current in shading_groups:
            mesh_shape = self.get_duplicate_mesh(mesh_object)
            if not mesh_shape:
                continue

            keep_faces = self.get_mesh_shading_members(current, mesh_shape)
            if not keep_faces:
                cmds.delete(cmds.listRelatives(mesh_shape, parent=True, fullPath=True))
                continue

            cmds.select(clear=True)
            cmds.select(keep_faces[0].split('.f')[0] + '.f[*]')
            cmds.select(keep_faces, deselect=True)
            cmds.delete()

            # CRITICAL! Assign the shading group to reset the connections
            cmds.sets(mesh_shape, edit=True, forceElement=current)

        cmds.undoInfo(state=undo_state)

    def material_break_button_pressed(self, *args):
        """Material break button event handler. Executes the script and breaks
        selected/all objects into parts based on material assignment""" 
        selection_only = cmds.checkBox(self.selection_only_checkbox, query=True, value=True)
        mesh_objects = self.get_mesh_objects(selection_only)
        error_counter = 0

        if not mesh_objects:
            cmds.text(self.status_label, edit=True, label='ERROR: No objects to operate on')
            return False
        
        cmds.progressBar(self.progress_bar, edit=True, progress=0, maxValue=(len(mesh_objects)))
        cmds.text(self.status_label, edit=True, label='INFO: Breaking objects. Please wait...')
        # cmds.refresh(currentView=True)
        
        for i in range(0, len(mesh_objects), 1):
            shading_groups = self.get_unique_shading_groups(mesh_objects[i])

            # Check mesh has at least two shading groups
            if len(shading_groups) <= 1:
                if not shading_groups:
                    print ('ERROR: Object ' + mesh_objects[i] + ' has no material assignment')
                    error_counter += 1
                continue

            self.set_material_break(mesh_objects[i], shading_groups)

            # CRITICAL! Use progress, step freezes with large lists
            cmds.progressBar(self.progress_bar, edit=True, progress=i)

        if not cmds.checkBox(self.keep_original_checkbox, query=True, value=True):
            cmds.delete(cmds.listRelatives(mesh_objects, parent=True, path=True))

        cmds.progressBar(self.progress_bar, edit=True, progress=(len(mesh_objects)))

        if error_counter == 0:
            cmds.text(self.status_label, edit=True, label='INFO: Operation complete')
        else:
            cmds.text(self.status_label, edit=True, label='WARNING: Check script editor')

    def setup_ui(self):
        """Creates the UI"""
        if cmds.window('MaterialBreakWindow', exists=True):
            cmds.deleteUI('MaterialBreakWindow')
        
        break_window = cmds.window('MaterialBreakWindow', title='Material Break',
                                   width=205, height=135, minimizeButton=False,
                                   maximizeButton=False, sizeable=False,
                                   resizeToFitChildren=True)
        cmds.columnLayout(columnWidth=200, adjustableColumn=False, rowSpacing=5)
        cmds.frameLayout(width=200, label='Options',
                         collapsable=False, marginHeight=5, marginWidth=5)
        self.keep_original_checkbox = cmds.checkBox(label='Keep Original Object', value=True)
        self.selection_only_checkbox = cmds.checkBox(label='Selection Only', value=True)
        cmds.setParent('..')  # end frameLayout
        cmds.columnLayout(columnWidth=200, adjustableColumn=False,
                          columnAttach=['left', 5], rowSpacing=5)
        self.progress_bar = cmds.progressBar(minValue=0, maxValue=100,
                                             width=188, height=10)
        self.material_break_button = cmds.button(label='Break Multi Material Objects',
                                                 width=188,
                                                 command=self.material_break_button_pressed)
        self.status_label = cmds.text(label='INFO: Standing by...', align='left')
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end columnLayout
        cmds.showWindow(break_window)


def main():
    """Main entry point. Opens the UI"""
    mb = MaterialBreak()
    mb.setup_ui()


if __name__ == '__main__':
    main()
